@@ -10,7 +10,6 @@ let d = r.division(&q);
 assert!((d * q + r - p).is_zero());
 ```
 */
-#![cfg_attr(feature = "__internal_inject_debug", recursion_limit = "8")]
 mod sealed {
     pub trait SizedExt: std::marker::Sized + std::fmt::Debug + std::fmt::Display {}
     impl<T> SizedExt for T where T: std::marker::Sized + std::fmt::Debug + std::fmt::Display {}
@@ -23,14 +22,9 @@ use modulo_n_tools::{add_mod, mul_mod, sub_mod};
 use num_traits::{One, Zero};
 use ring_algorithm::{modulo_inverse, RingNormalize};
 use sealed::Sized;
-use std::ops::{Add, AddAssign, Mul, Neg, Rem, Sub, SubAssign};
+use std::ops::{Add, AddAssign, Div, Mul, Neg, Rem, Sub, SubAssign};
 mod ops;
 
-#[derive(Clone, Debug, PartialEq, Eq)]
-pub struct PolynomialOverP<T> {
-    coef: Vec<T>,
-    prime: T,
-}
 /** PolynomialOverP ring over finite prime field $`F_p[x]`$
 
 ```
@@ -43,6 +37,38 @@ let d = r.division(&q);
 assert!((d * q + r - p).is_zero());
 ```
 */
+#[derive(Clone, Debug)]
+pub struct PolynomialOverP<T> {
+    coef: Vec<T>,
+    prime: T,
+}
+
+impl<T: Sized> PartialEq for PolynomialOverP<T>
+where
+    T: Clone + Eq + Zero,
+    for<'x> &'x T: Sub<Output = T> + Rem<Output = T>,
+{
+    fn eq(&self, other: &Self) -> bool {
+        if self.prime != other.prime {
+            return false;
+        }
+        if self.coef.len() != other.coef.len() {
+            return false;
+        }
+        for (v, w) in self.coef.iter().zip(other.coef.iter()) {
+            if !(&(v - w) % &self.prime).is_zero() {
+                return false;
+            }
+        }
+        true
+    }
+}
+impl<T: Sized> Eq for PolynomialOverP<T>
+where
+    T: Clone + Eq + Zero,
+    for<'x> &'x T: Sub<Output = T> + Rem<Output = T>,
+{
+}
 
 impl<T: Sized> PolynomialOverP<T> {
     fn len(&self) -> usize {
@@ -85,7 +111,7 @@ impl<T: Sized> PolynomialOverP<T> {
     let p = PolynomialOverP::<i32>::new(vec![3, 2, 1], 5); // 3+2x+x^2
     assert_eq!(p.coefs(), vec![3, 2, 1]);
     let q = PolynomialOverP::<i32>::new(vec![0], 5); // 0
-    assert_eq!(q.coefs(), vec![]);
+    assert_eq!(q.coefs(), Vec::<i32>::new());
     ```
     */
     pub fn coefs(self) -> Vec<T> {
@@ -172,7 +198,7 @@ impl<T: Sized> PolynomialOverP<T> {
     fn add_assign_ref(&mut self, other: &Self)
     where
         T: Clone + Ord + Zero + for<'x> AddAssign<&'x T> + for<'x> SubAssign<&'x T>,
-        for<'x> &'x T: Add<Output = T> + Neg<Output = T>,
+        for<'x> &'x T: Add<Output = T> + Sub<Output = T> + Neg<Output = T> + Rem<Output = T>,
     {
         if self.is_zero() {
             *self = other.clone();
@@ -215,7 +241,7 @@ impl<T: Sized> PolynomialOverP<T> {
     fn sub_assign_ref(&mut self, other: &Self)
     where
         T: Clone + Ord + Zero + for<'x> AddAssign<&'x T> + for<'x> SubAssign<&'x T>,
-        for<'x> &'x T: Add<Output = T> + Sub<Output = T> + Neg<Output = T>,
+        for<'x> &'x T: Add<Output = T> + Sub<Output = T> + Neg<Output = T> + Rem<Output = T>,
     {
         if self.is_zero() {
             *self = -other;
@@ -237,7 +263,8 @@ impl<T: Sized> PolynomialOverP<T> {
     fn mul_impl(&self, other: &Self) -> Self
     where
         T: Sized + Clone + Ord + Zero + for<'x> AddAssign<&'x T> + for<'x> SubAssign<&'x T>,
-        for<'x> &'x T: Add<Output = T> + Neg<Output = T> + Mul<Output = T> + Rem<Output = T>,
+        for<'x> &'x T:
+            Add<Output = T> + Sub<Output = T> + Neg<Output = T> + Mul<Output = T> + Rem<Output = T>,
     {
         if self.is_zero() || other.is_zero() {
             let prime = if self.prime.is_zero() {
@@ -309,6 +336,7 @@ impl<T: Sized> PolynomialOverP<T> {
     assert_eq!(q.derivative(), PolynomialOverP::<usize>::new(vec![2, 1, 2, 2, 0, 1], 5));
     ```
     */
+    #[must_use]
     pub fn derivative(self) -> Self
     where
         T: Clone + Zero + for<'x> AddAssign<&'x T> + From<usize>,
@@ -336,7 +364,8 @@ impl<T: Sized> PolynomialOverP<T> {
     pub fn monic(&mut self)
     where
         T: Clone + Eq + Zero + One + RingNormalize,
-        for<'x> &'x T: ring_algorithm::EuclideanRingOperation<T>,
+        for<'x> &'x T:
+            Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Div<Output = T> + Rem<Output = T>,
     {
         if let Some(lc) = self.lc() {
             let prime = self.prime.clone();
@@ -358,6 +387,7 @@ impl<T: Sized> PolynomialOverP<T> {
     assert!((d * q + r - p).is_zero());
     ```
     */
+    #[allow(unknown_lints, clippy::return_self_not_must_use)]
     pub fn division(&mut self, other: &Self) -> Self
     where
         T: Clone
@@ -368,7 +398,8 @@ impl<T: Sized> PolynomialOverP<T> {
             + for<'x> AddAssign<&'x T>
             + for<'x> SubAssign<&'x T>
             + RingNormalize,
-        for<'x> &'x T: ring_algorithm::EuclideanRingOperation<T> + Neg<Output = T>,
+        for<'x> &'x T:
+            Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Div<Output = T> + Rem<Output = T>,
     {
         let g_deg = other.deg().expect("Division by zero");
         if self.deg() < other.deg() {
@@ -449,7 +480,8 @@ where
 impl<T> RingNormalize for PolynomialOverP<T>
 where
     T: Sized + Clone + Eq + Zero + One + RingNormalize,
-    for<'x> &'x T: ring_algorithm::EuclideanRingOperation<T>,
+    for<'x> &'x T:
+        Add<Output = T> + Sub<Output = T> + Mul<Output = T> + Div<Output = T> + Rem<Output = T>,
 {
     fn leading_unit(&self) -> Self {
         if let Some(x) = self.lc() {
@@ -466,7 +498,7 @@ where
 impl<T> Zero for PolynomialOverP<T>
 where
     T: Sized + Clone + Ord + Zero + for<'x> AddAssign<&'x T> + for<'x> SubAssign<&'x T>,
-    for<'x> &'x T: Add<Output = T> + Neg<Output = T>,
+    for<'x> &'x T: Add<Output = T> + Sub<Output = T> + Neg<Output = T> + Rem<Output = T>,
 {
     fn zero() -> Self {
         Self {
